@@ -1,31 +1,127 @@
 """Provides the O&M Enviroment class; a subclass of simpy.Environment."""
 from __future__ import annotations
 
-import math  # type: ignore
-import logging  # type: ignore
-import datetime as dt  # type: ignore
-from typing import Tuple, Union, Optional  # type: ignore
-from pathlib import Path  # type: ignore
-from datetime import datetime, timedelta  # type: ignore
+import csv
+import math
+import logging
+import datetime as dt
+from typing import TYPE_CHECKING, Tuple, Union, Optional
+from pathlib import Path
+from datetime import datetime, timedelta
 
-import numpy as np  # type: ignore
-import simpy  # type: ignore
-import pandas as pd  # type: ignore
-from simpy.events import Event  # type: ignore
+import numpy as np
+import simpy
+import pandas as pd
+import pyarrow as pa
+import pyarrow.csv  # pylint: disable=W0611
+from simpy.events import Event
 from pandas.core.indexes.datetimes import DatetimeIndex
 
 import wombat  # pylint: disable=W0611
-from wombat.utilities import (
-    setup_logger,
-    hours_until_future_hour,
-    format_events_log_message,
-)
+from wombat.utilities import hours_until_future_hour
+from wombat.core.data_classes import parse_date
+
+
+if TYPE_CHECKING:
+    from wombat.windfarm import Windfarm
+
+
+EVENTS_COLUMNS = [
+    "datetime",
+    "env_datetime",
+    "env_time",
+    "agent",
+    "action",
+    "reason",
+    "additional",
+    "system_id",
+    "system_name",
+    "part_id",
+    "part_name",
+    "system_operating_level",
+    "part_operating_level",
+    "duration",
+    "distance_km",
+    "request_id",
+    "location",
+    "materials_cost",
+    "hourly_labor_cost",
+    "salary_labor_cost",
+    "total_labor_cost",
+    "equipment_cost",
+    "total_cost",
+]
 
 
 class WombatEnvironment(simpy.Environment):
     """The primary mechanism for powering an O&M simulation. This object has insight into
     all other simulation objects, and controls the timing, date/time stamps, and weather
     conditions.
+
+    Parameters
+    ----------
+    data_dir : pathlib.Path | str
+        Directory where the inputs are stored and where to save outputs.
+    weather_file : str
+        Name of the weather file. Should be contained within ``data_dir``/weather/, with
+        columns "datetime", "windspeed", and, optionally, "waveheight". The datetime
+        column should adhere to the following format: "MM/DD/YY HH:MM", in 24-hour time.
+    workday_start : int
+        Starting time for the repair crew, in 24 hour local time. This can be overridden
+        by an ``ServiceEquipmentData`` object that operates outside of the "typical"
+        working hours.
+    workday_end : int
+        Ending time for the repair crew, in 24 hour local time. This can be overridden
+        by an ``ServiceEquipmentData`` object that operates outside of the "typical"
+        working hours.
+    simulation_name : str | None, optional
+        Name of the simulation; will be used for naming the log file, by default ``None``.
+        If ``None``, then the current time will be used. Will always save to
+        ``data_dir``/outputs/logs/``simulation_name``.log.
+
+        .. note: spaces (" ") will be replaced with underscores ("_"), for example:
+            "my example analysis" becomes "my_example_analysis".
+
+    start_year : int | None, optional
+        Custom starting year for the weather profile, by default None. If ``None`` or
+        less than the first year of the weather profile, this will be ignored.
+    end_year : int | None, optional
+        Custom ending year for the weather profile, by default None. If ``None`` or
+        greater than the last year of the weather profile, this will be ignored.
+    port_distance : int | float
+        The simulation-wide daily travel distance for servicing equipment. This
+        should be used as a base setting when multiple or all servicing equipment
+        will be operating out of the same base location, but can be individually
+        modified.
+    non_operational_start : str | datetime.datetime | None
+        The starting month and day, e.g., MM/DD, M/D, MM-DD, etc. for an annualized
+        period of prohibited operations. When defined at the environment level,
+        an undefined or later starting date will be overridden for all servicing
+        equipment and any modeled port, by default None.
+    non_operational_end : str | datetime.datetime | None
+        The ending month and day, e.g., MM/DD, M/D, MM-DD, etc. for an annualized
+        period of prohibited operations. When defined at the environment level,
+        an undefined or earlier ending date will be overridden for all servicing
+        equipment and any modeled port, by default None.
+    reduced_speed_start : str | datetime.datetime | None
+        The starting month and day, e.g., MM/DD, M/D, MM-DD, etc. for an annualized
+        period of reduced speed operations. When defined at the environment level,
+        an undefined or later starting date will be overridden for all servicing
+        equipment and any modeled port, by default None.
+    reduced_speed_end : str | datetime.datetime | None
+        The ending month and day, e.g., MM/DD, M/D, MM-DD, etc. for an annualized
+        period of reduced speed operations. When defined at the environment level,
+        an undefined or earlier ending date will be overridden for all servicing
+        equipment and any modeled port, by default None.
+    reduced_speed : float
+        The maximum operating speed during the annualized reduced speed operations.
+        When defined at the environment level, an undefined or faster value will be
+        overridden for all servicing equipment and any modeled port, by default 0.0.
+
+    Raises
+    ------
+    FileNotFoundError
+        Raised if ``data_dir`` cannot be found.
     """
 
     def __init__(
@@ -37,41 +133,14 @@ class WombatEnvironment(simpy.Environment):
         simulation_name: str | None = None,
         start_year: int | None = None,
         end_year: int | None = None,
+        port_distance: int | float | None = None,
+        non_operational_start: str | dt.datetime | None = None,
+        non_operational_end: str | dt.datetime | None = None,
+        reduced_speed_start: str | dt.datetime | None = None,
+        reduced_speed_end: str | dt.datetime | None = None,
+        reduced_speed: float = 0.0,
     ) -> None:
-        """Initialization.
-
-        Parameters
-        ----------
-        data_dir : pathlib.Path | str
-            Directory where the inputs are stored and where to save outputs.
-        weather_file : str
-            Name of the weather file. Should be contained within ``data_dir``/weather/.
-        workday_start : int
-            Starting time for the repair crew, in 24 hour local time. This can be
-            overridden by an ``ServiceEquipmentData`` object that operates outside of the "typical"
-            working hours.
-        workday_end : int
-            Ending time for the repair crew, in 24 hour local time. This can be
-            overridden by an ``ServiceEquipmentData`` object that operates outside of the "typical"
-            working hours.
-        simulation_name : str | None, optional
-            Name of the simulation; will be used for naming the log file, by default ``None``.
-            If ``None``, then the current time will be used. Will always save to
-            ``data_dir``/outputs/logs/``simulation_name``.log.
-            ... note: spaces (" ") will be replaced with underscores ("_"), for example:
-            "my example analysis" becomes "my_example_analysis".
-        start_year : int | None, optional
-            Custom starting year for the weather profile, by default None. If ``None`` or
-            less than the first year of the weather profile, this will be ignored.
-        end_year : int | None, optional
-            Custom ending year for the weather profile, by default None. If ``None`` or
-            greater than the last year of the weather profile, this will be ignored.
-
-        Raises
-        ------
-        FileNotFoundError
-            Raised if ``data_dir`` cannot be found.
-        """
+        """Initialization."""
         super().__init__()
         self.data_dir = Path(data_dir).resolve()
         if not self.data_dir.is_dir():
@@ -79,21 +148,35 @@ class WombatEnvironment(simpy.Environment):
 
         self.workday_start = int(workday_start)
         self.workday_end = int(workday_end)
-        if not 0 <= self.workday_start < 24:
-            raise ValueError("workday_start must be a valid 24hr time before midnight!")
-        if not 0 <= self.workday_end < 24:
-            raise ValueError("workday_end must be a valid 24hr time before midnight!")
+        if not 0 <= self.workday_start <= 24:
+            raise ValueError("workday_start must be a valid 24hr time before midnight.")
+        if not 0 <= self.workday_end <= 24:
+            raise ValueError("workday_end must be a valid 24hr time.")
         if self.workday_end <= self.workday_start:
             raise ValueError(
                 "Work shifts must end after they start ({self.workday_start}hrs)."
             )
 
+        self.port_distance = port_distance
         self.weather = self._weather_setup(weather_file, start_year, end_year)
+        self.weather_dates = self.weather.index.to_pydatetime()
         self.max_run_time = self.weather.shape[0]
         self.shift_length = self.workday_end - self.workday_start
 
+        # Set the environmental consideration parameters
+        self.non_operational_start = parse_date(non_operational_start)
+        self.non_operational_end = parse_date(non_operational_end)
+        self.reduced_speed_start = parse_date(reduced_speed_start)
+        self.reduced_speed_end = parse_date(reduced_speed_end)
+        self.reduced_speed = reduced_speed
+
         self.simulation_name = simulation_name
         self._logging_setup()
+        self.process(self._log_actions())
+
+    def _register_windfarm(self, windfarm: Windfarm) -> None:
+        """Adds the simulation windfarm to the class attributes"""
+        self.windfarm = windfarm
 
     def run(self, until: Optional[Union[int, float, Event]] = None):
         """Extends the ``simpy.Environment.run`` method to change the default behavior if
@@ -106,6 +189,18 @@ class WombatEnvironment(simpy.Environment):
             When to stop the simulation, by default None. See documentation on
             ``simpy.Environment.run`` for more details.
         """
+        # If running a paused simulation, then reopen the file and append, but only if
+        # the simulation time is lower than the upper bound
+        time_check = self.now < self.max_run_time
+        if self._events_csv.closed and time_check:  # type: ignore
+            self._events_csv = open(self.events_log_fname, "a")
+            self._events_writer = csv.DictWriter(
+                self._events_csv, delimiter="|", fieldnames=EVENTS_COLUMNS
+            )
+        if hasattr(self, "windfarm") and self._operations_csv.closed and time_check:  # type: ignore
+            self._operations_csv = open(self.operations_log_fname, "a")
+            self.windfarm._setup_logger(initial=False)
+
         if until is None:
             until = self.max_run_time
         elif until > self.max_run_time:
@@ -114,13 +209,21 @@ class WombatEnvironment(simpy.Environment):
             super().run(until=until)
         except BaseException as e:
             # Flush the logs to so the simulation up to the point of failure is logged
-            self._events_logger.handlers[0].flush()
-            self._operations_logger.handlers[0].flush()
+            self._events_writer.writerows(self._events_buffer)
+            self._events_buffer.clear()
+            self._events_csv.close()
+            self._operations_writer.writerows(self._operations_buffer)
+            self._operations_buffer.clear()
+            self._operations_csv.close()
             raise e
 
         # Ensure all logged events make it to their target file
-        self._events_logger.handlers[0].flush()
-        self._operations_logger.handlers[0].flush()
+        self._events_writer.writerows(self._events_buffer)
+        self._events_buffer.clear()
+        self._events_csv.close()
+        self._operations_writer.writerows(self._operations_buffer)
+        self._operations_buffer.clear()
+        self._operations_csv.close()
 
     def _logging_setup(self) -> None:
         """Completes the setup for logging data."""
@@ -128,29 +231,36 @@ class WombatEnvironment(simpy.Environment):
             self.simulation_name = simulation = "wombat"
         else:
             simulation = self.simulation_name.replace(" ", "_")
-        dt_stamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S.%f")
+        dt_stamp = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
 
-        events_log_fname = f"{dt_stamp}_{simulation}_events.log"
-        operations_log_fname = f"{dt_stamp}_{simulation}_operations.log"
+        events_log_fname = f"{dt_stamp}_{simulation}_events.csv"
+        operations_log_fname = f"{dt_stamp}_{simulation}_operations.csv"
         power_potential_fname = f"{dt_stamp}_{simulation}_power_potential.csv"
         power_production_fname = f"{dt_stamp}_{simulation}_power_production.csv"
         metrics_input_fname = f"{dt_stamp}_{simulation}_metrics_inputs.yaml"
 
-        log_path = self.data_dir / "outputs" / "logs"
+        log_path = self.data_dir / "results"
+        if not log_path.exists():
+            log_path.mkdir()
         self.events_log_fname = log_path / events_log_fname
         self.operations_log_fname = log_path / operations_log_fname
         self.power_potential_fname = log_path / power_potential_fname
         self.power_production_fname = log_path / power_production_fname
         self.metrics_input_fname = log_path / metrics_input_fname
 
-        _dir = self.data_dir / "outputs" / "logs"
+        _dir = self.data_dir / "results"
         if not _dir.is_dir():
             _dir.mkdir()
 
-        setup_logger("events_log", self.events_log_fname, capacity=7000)
-        setup_logger("operations_log", self.operations_log_fname, capacity=7000)
-        self._events_logger = logging.getLogger("events_log")
-        self._operations_logger = logging.getLogger("operations_log")
+        self._events_csv = open(self.events_log_fname, "w")
+        self._operations_csv = open(self.operations_log_fname, "w")
+        self._events_writer = csv.DictWriter(
+            self._events_csv, delimiter="|", fieldnames=EVENTS_COLUMNS
+        )
+        self._events_writer.writeheader()
+
+        self._events_buffer: list[dict] = []
+        self._operations_buffer: list[dict] = []
 
     @property
     def simulation_time(self) -> datetime:
@@ -158,10 +268,10 @@ class WombatEnvironment(simpy.Environment):
         now = self.now
         minutes = now % 1 * 60
         if now == self.max_run_time:
-            _dt = self.weather.iloc[math.floor(now - 1)].name.to_pydatetime()
+            _dt = self.weather_dates[math.floor(now - 1)]
             _dt + timedelta(hours=1)
         else:
-            _dt = self.weather.iloc[math.floor(now)].name.to_pydatetime()
+            _dt = self.weather_dates[math.floor(now)]
 
         minutes, seconds = math.floor(minutes), math.ceil(minutes % 1 * 60)
         return _dt + timedelta(minutes=minutes, seconds=seconds)
@@ -184,7 +294,7 @@ class WombatEnvironment(simpy.Environment):
             True if it's valid working hours, False otherwise.
         """
         if -1 in (workday_start, workday_end):
-            # Return true if the shift is around the clock
+            # Return True if the shift is around the clock
             if self.workday_start == 0 and self.workday_end == 24:
                 return True
             return self.workday_start <= self.simulation_time.hour < self.workday_end
@@ -265,9 +375,12 @@ class WombatEnvironment(simpy.Environment):
         int
             Index of the weather profile corresponds to the first hour of ``date``.
         """
-        if isinstance(date, dt.datetime):
-            date = date.date()
-        return np.where(self.weather.index.date == date)[0][0]
+        ix = self.weather.index.get_loc(date.strftime("%Y-%m-%d"))
+
+        # If the index is consecutive a slice is returned, else a numpy array
+        if isinstance(ix, slice):
+            return ix.start
+        return ix[0]
 
     def _weather_setup(
         self,
@@ -298,13 +411,29 @@ class WombatEnvironment(simpy.Environment):
         pd.DataFrame
             The wind (and  wave) timeseries.
         """
-        weather_path = self.data_dir / "weather" / weather_file
-        weather = pd.read_csv(
-            weather_path, parse_dates=["datetime"], index_col="datetime"
+        # PyArrow datetime conversion setup
+        convert_options = pa.csv.ConvertOptions(
+            timestamp_parsers=[
+                "%m/%d/%y %H:%M",
+                "%m/%d/%y %I:%M",
+                "%m/%d/%Y %H:%M",
+                "%m/%d/%Y %I:%M",
+            ]
+        )
+        weather = (
+            pa.csv.read_csv(
+                self.data_dir / "weather" / weather_file,
+                convert_options=convert_options,
+            )
+            .to_pandas()
+            .set_index("datetime")
         )
         weather = weather.fillna(0.0)
         weather = weather.resample("H").interpolate(limit_direction="both", limit=5)
 
+        # Add in the hour of day column for more efficient handling within the simulation
+        weather = weather.assign(hour=weather.index.hour.astype(float))
+
         # Create the start and end points
         self.start_datetime = weather.index[0].to_pydatetime()
         self.end_datetime = weather.index[-1].to_pydatetime()
@@ -318,12 +447,12 @@ class WombatEnvironment(simpy.Environment):
             pass
         elif start_year > self.end_year:
             raise ValueError(
-                f"'stary_year' ({start_year}) occurs after the last available year"
+                f"'start_year' ({start_year}) occurs after the last available year"
                 f" in the weather data (range: {self.end_year})"
             )
         else:
             # Filter for the provided, validated starting year and update the attribute
-            weather = weather[weather.index.year >= start_year]
+            weather = weather.loc[weather.index.year >= start_year]
             self.start_datetime = weather.index[0].to_pydatetime()
             start_year = self.start_year = self.start_datetime.year
 
@@ -342,25 +471,25 @@ class WombatEnvironment(simpy.Environment):
                 )
             else:
                 # Filter for the provided, validated ending year and update the attribute
-                weather = weather[weather.index.year <= end_year]
+                weather = weather.loc[weather.index.year <= end_year]
                 self.end_datetime = weather.index[-1].to_pydatetime()
                 self.end_year = self.end_datetime.year
         else:
             # Filter for the provided, validated ending year and update the attribute
-            weather = weather[weather.index.year <= end_year]
+            weather = weather.loc[weather.index.year <= end_year]
             self.end_datetime = weather.index[-1].to_pydatetime()
             self.end_year = self.end_datetime.year
 
         return weather
 
     @property
-    def weather_now(self) -> Tuple[float, float]:
+    def weather_now(self) -> Tuple[float, float, int]:
         """The current weather.
 
         Returns
         -------
-        Tuple[float, float]
-            Wind and wave data for the current time.
+        Tuple[float, float, int]
+            Wind, wave, and hour data for the current time.
         """
         # Rounds down because we won't arrive at the next weather event until that hour
         now = int(self.now)
@@ -368,7 +497,7 @@ class WombatEnvironment(simpy.Environment):
 
     def weather_forecast(
         self, hours: Union[int, float]
-    ) -> Tuple[DatetimeIndex, np.ndarray, np.ndarray]:
+    ) -> Tuple[DatetimeIndex, np.ndarray, np.ndarray, np.ndarray]:
         """Returns the wind and wave data for the next ``hours`` hours, starting from
         the current hour's weather.
 
@@ -379,19 +508,18 @@ class WombatEnvironment(simpy.Environment):
 
         Returns
         -------
-        Tuple[DatetimeIndex, np.ndarray, np.ndarray]
+        Tuple[DatetimeIndex, np.ndarray, np.ndarray, np.ndarray]
             The pandas DatetimeIndex, windspeed array, and waveheight array for the
             hours requested, each with shape (``hours`` + 1).
         """
         start = math.floor(self.now)
-        end = start + math.ceil(hours)
 
         # If it's not on the hour, ensure we're looking ``hours`` hours into the future
-        if self.now % 1:
-            end += 1
+        end = start + math.ceil(hours) + math.ceil(self.now % 1)
 
-        weather = self.weather.iloc[start:end]
-        return (weather.index, weather.windspeed.values, weather.waveheight.values)
+        wind, wave, hour = self.weather.values[start:end].T
+        ix = self.weather.index[start:end]
+        return ix, hour, wind, wave
 
     def log_action(
         self,
@@ -407,6 +535,7 @@ class WombatEnvironment(simpy.Environment):
         system_ol: float | int = 0,
         part_ol: float | int = 0,
         duration: float = 0,
+        distance_km: float = 0,
         request_id: str = "na",
         location: str = "na",
         materials_cost: Union[int, float] = 0,
@@ -446,6 +575,8 @@ class WombatEnvironment(simpy.Environment):
             enroute, or system, by default "na".
         duration : float
             Length of time the action lasted, by default 0.
+        distance : float
+            Distance traveled, in km, if applicable, by default 0.
         materials_cost : Union[int, float], optional
             Total cost of materials for action, in USD, by default 0.
         hourly_labor_cost : Union[int, float], optional
@@ -460,31 +591,44 @@ class WombatEnvironment(simpy.Environment):
             raise ValueError(
                 f"Event logging `location` must be one of: {valid_locations}"
             )
-        self._events_logger.info(
-            format_events_log_message(
-                self.simulation_time,
-                self.now,
-                system_id,
-                system_name,
-                part_id,
-                part_name,
-                system_ol,
-                part_ol,
-                agent,
-                action,
-                reason,
-                additional,
-                duration,
-                request_id,
-                location,
-                materials_cost,
-                hourly_labor_cost,
-                salary_labor_cost,
-                equipment_cost,
-            )
+        total_labor_cost = hourly_labor_cost + salary_labor_cost
+        total_cost = total_labor_cost + equipment_cost + materials_cost
+        row = dict(
+            datetime=dt.datetime.now(),
+            env_datetime=self.simulation_time,
+            env_time=self.now,
+            system_id=system_id,
+            system_name=system_name,
+            part_id=part_id,
+            part_name=part_name,
+            system_operating_level=system_ol,
+            part_operating_level=part_ol,
+            agent=agent,
+            action=action,
+            reason=reason,
+            additional=additional,
+            duration=duration,
+            distance_km=distance_km,
+            request_id=request_id,
+            location=location,
+            materials_cost=materials_cost,
+            hourly_labor_cost=hourly_labor_cost,
+            salary_labor_cost=salary_labor_cost,
+            equipment_cost=equipment_cost,
+            total_labor_cost=total_labor_cost,
+            total_cost=total_cost,
         )
+        self._events_buffer.append(row)
 
-    def _create_events_log_dataframe(self) -> pd.DataFrame:
+    def _log_actions(self):
+        """Writes the action log items every 8000 hours"""
+        HOURS = 8000
+        while True:
+            yield self.timeout(HOURS)
+            self._events_writer.writerows(self._events_buffer)
+            self._events_buffer.clear()
+
+    def load_events_log_dataframe(self) -> pd.DataFrame:
         """Imports the logging file created in ``run`` and returns it as a formatted
         ``pandas.DataFrame``.
 
@@ -493,48 +637,22 @@ class WombatEnvironment(simpy.Environment):
         pd.DataFrame
             The formatted logging data from a simulation.
         """
-        log_columns = [
-            "datetime",
-            "name",
-            "level",
-            "env_datetime",
-            "env_time",
-            "system_id",
-            "system_name",
-            "part_id",
-            "part_name",
-            "system_operating_level",
-            "part_operating_level",
-            "agent",
-            "action",
-            "reason",
-            "additional",
-            "duration",
-            "request_id",
-            "location",
-            "materials_cost",
-            "hourly_labor_cost",
-            "salary_labor_cost",
-            "equipment_cost",
-            "total_labor_cost",
-            "total_cost",
-        ]
-        log_df = pd.read_csv(
+        convert_options = pa.csv.ConvertOptions(
+            timestamp_parsers=["%y-%m-%d %H:%M:%S.%f", "%y-%m-%d %H:%M:%S"]
+        )
+        parse_options = pa.csv.ParseOptions(delimiter="|")
+        log_df = pa.csv.read_csv(
             self.events_log_fname,
-            names=log_columns,
-            sep=" :: ",
-            index_col=False,
-            engine="python",
-        )
-        log_df = log_df.loc[log_df.level == "INFO"]
-        log_df = log_df.drop(labels=["name", "level"], axis=1)
+            convert_options=convert_options,
+            parse_options=parse_options,
+        ).to_pandas()
         log_df.datetime = pd.to_datetime(log_df.datetime)
         log_df.env_datetime = pd.to_datetime(log_df.env_datetime)
-        log_df.index = log_df.datetime
-        log_df = log_df.drop(labels="datetime", axis=1)
+        log_df = log_df.set_index("datetime").sort_values("datetime")
+
         return log_df
 
-    def _create_operations_log_dataframe(self) -> pd.DataFrame:
+    def load_operations_log_dataframe(self) -> pd.DataFrame:
         """Imports the logging file created in ``run`` and returns it as a formatted
         ``pandas.DataFrame``.
 
@@ -543,62 +661,21 @@ class WombatEnvironment(simpy.Environment):
         pd.DataFrame
             The formatted logging data from a simulation.
         """
-        with open(self.operations_log_fname) as f:
-            first_line = f.readline().strip().split(" :: ")
-            column_names = first_line[3:]
-
-        log_df = pd.read_csv(
-            self.operations_log_fname,
-            names=column_names,
-            skiprows=1,
-            sep=" :: ",
-            index_col=False,
-            engine="python",
+        convert_options = pa.csv.ConvertOptions(
+            timestamp_parsers=["%y-%m-%d %H:%M:%S.%f", "%y-%m-%d %H:%M:%S"]
         )
-
-        log_df = log_df.loc[log_df.level == "INFO"]
-        log_df = log_df.drop(labels=["name", "level"], axis=1)
+        parse_options = pa.csv.ParseOptions(delimiter="|")
+        log_df = pa.csv.read_csv(
+            self.operations_log_fname,
+            convert_options=convert_options,
+            parse_options=parse_options,
+        ).to_pandas()
         log_df.datetime = pd.to_datetime(log_df.datetime)
         log_df.env_datetime = pd.to_datetime(log_df.env_datetime)
-        log_df.index = log_df.datetime
-        log_df = log_df.drop(labels="datetime", axis=1)
+        log_df = log_df.set_index("datetime").sort_values("datetime")
+
         return log_df
 
-    def convert_logs_to_csv(  # type: ignore
-        self, delete_original: bool = False, return_df: bool = True
-    ) -> Tuple[pd.DataFrame, pd.DataFrame]:
-        """Creates a CSV file for the both the events and operations logs.
-
-        Parameters
-        ----------
-        delete_original : bool, optional
-            If True, delete the corresponding ".log" files, by default False.
-        return_df : bool, optional
-            If True, returns the pd.DataFrame objects for the operations and events
-            logging, by default True.
-
-        Returns
-        -------
-        Tuple[pd.DataFrame, pd.DataFrame]
-            Returns nothing if ``return_df`` is False, otherwise returns the operations and
-            events log ``pd.DataFrame`` objects in that order.
-        """
-        events = self._create_events_log_dataframe().sort_values("env_time")
-        operations = self._create_operations_log_dataframe().sort_values("env_time")
-
-        events_fname = self.events_log_fname.with_suffix(".csv")
-        events.to_csv(events_fname, index=False)
-
-        operations_fname = self.operations_log_fname.with_suffix(".csv")
-        operations.to_csv(operations_fname, index=False)
-
-        if delete_original:
-            self.operations_log_fname.unlink()
-            self.events_log_fname.unlink()
-
-        if return_df:
-            return operations, events
-
     def power_production_potential_to_csv(  # type: ignore
         self,
         windfarm: "wombat.windfarm.Windfarm",  # type: ignore
@@ -613,7 +690,8 @@ class WombatEnvironment(simpy.Environment):
             The simulation's windfarm object.
         operations : Optional[pd.DataFrame], optional
             The operations log ``DataFrame`` if readily available, by default None. If
-            ``None``, then it will be created through ``convert_logs_to_csv()``.
+            ``None``, then it will be created through
+            ``load_operations_log_dataframe()``.
         return_df : bool, optional
             Indicator to return the power production for further usage, by default True.
 
@@ -622,53 +700,66 @@ class WombatEnvironment(simpy.Environment):
         Tuple[pd.DataFrame, pd.DataFrame]
             The power potential and production timeseries data.
         """
+        write_options = pa.csv.WriteOptions(delimiter="|")
+
         if operations is None:
-            operations = self._create_operations_log_dataframe().sort_values("env_time")
+            operations = self.load_operations_log_dataframe().sort_values("env_time")
 
         turbines = windfarm.turbine_id
         windspeed = self.weather.windspeed
-        windspeed = windspeed.loc[operations.env_datetime]
-
-        potential = np.vstack(
-            ([windfarm.system(t_id).power(windspeed) for t_id in turbines])
-        ).T
+        windspeed = windspeed.loc[operations.env_datetime].values
         potential_df = pd.DataFrame(
             [],
-            index=windspeed.index,
+            index=operations.env_datetime,
             columns=["env_time", "env_datetime", "windspeed", "windfarm"]
             + turbines.tolist(),
         )
-        potential_df[turbines] = potential
-        potential_df.windspeed = windspeed.values
-        potential_df.windfarm = potential.sum(axis=1)
-        potential_df.env_time = operations.env_time.values
-        potential_df.env_datetime = operations.env_datetime.values
-        potential_df.to_csv(self.power_potential_fname, index_label="env_datetime")
-
-        production_df = potential_df.copy()
-        production_df[turbines] = (
-            production_df[turbines].values * operations[turbines].values
+        potential_df[turbines] = np.vstack(
+            ([windfarm.system(t_id).power(windspeed) for t_id in turbines])
+        ).T
+        potential_df = potential_df.assign(
+            windspeed=windspeed,
+            windfarm=potential_df[turbines].sum(axis=1),
+            env_time=operations.env_time.values,
+            env_datetime=operations.env_datetime.values,
         )
+        pa.csv.write_csv(
+            pa.Table.from_pandas(potential_df),
+            self.power_potential_fname,
+            write_options=write_options,
+        )
+
+        # TODO: The actual windfarm production needs to be clipped at each subgraph to
+        # the max of the substation's operating capacity and then summed.
+        production_df = potential_df.copy()
+        production_df[turbines] *= operations[turbines].values
         production_df.windfarm = production_df[turbines].sum(axis=1)
-        production_df.to_csv(self.power_production_fname, index_label="env_datetime")
+        pa.csv.write_csv(
+            pa.Table.from_pandas(production_df),
+            self.power_production_fname,
+            write_options=write_options,
+        )
         if return_df:
             return potential_df, production_df
 
-    def cleanup_log_files(self, log_only=False) -> None:
+    def cleanup_log_files(self) -> None:
         """This is a convenience method to clear the output log files in case a large
         batch of simulations is being run and there are space limitations.
 
-        Parameters
-        ----------
-        log_only : bool, optional
-            Only deletes the xx.log files, if True, otherwise the xx.log and xx.csv
-            logging files are all deleted, by default False
+        ... warning:: This shuts down the loggers, so no more logging will be able
+            to be performed.
         """
 
         # NOTE: Everything is wrapped in a try/except clause to protect against failure
         # when inevitably a file has already been deleted on accident, or if in the
         # dataframe generation step, the original logs were deleted
 
+        logging.shutdown()
+        if not self._events_csv.closed:
+            self._events_csv.close()
+        if not self._operations_csv.closed:
+            self._operations_csv.close()
+
         try:
             self.events_log_fname.unlink()
         except FileNotFoundError:
@@ -679,29 +770,17 @@ class WombatEnvironment(simpy.Environment):
         except FileNotFoundError:
             pass
 
-        if not log_only:
-            # Don't fail if any of the following files were not created
-            try:
-                self.events_log_fname.with_suffix(".csv").unlink()
-            except FileNotFoundError:
-                pass
+        try:
+            self.power_potential_fname.unlink()
+        except FileNotFoundError:
+            pass
 
-            try:
-                self.operations_log_fname.with_suffix(".csv").unlink()
-            except FileNotFoundError:
-                pass
+        try:
+            self.power_production_fname.unlink()
+        except FileNotFoundError:
+            pass
 
-            try:
-                self.power_potential_fname.unlink()
-            except FileNotFoundError:
-                pass
-
-            try:
-                self.power_production_fname.unlink()
-            except FileNotFoundError:
-                pass
-
-            try:
-                self.metrics_input_fname.unlink()
-            except FileNotFoundError:
-                pass
+        try:
+            self.metrics_input_fname.unlink()
+        except FileNotFoundError:
+            pass
